@@ -6,7 +6,7 @@ import collections
 
 from marshmallow.compat import basestring
 
-from marshmallow import ValidationError, class_registry
+from marshmallow import ValidationError, class_registry, missing
 # Make core fields importable from marshmallow_jsonapi
 from marshmallow.fields import *  # noqa
 from marshmallow.base import SchemaABC
@@ -101,7 +101,7 @@ class Relationship(BaseRelationship):
 
     @property
     def schema(self):
-        only = getattr(self, 'only', ())
+        only = getattr(self, 'only', None)
         exclude = getattr(self, 'exclude', ())
 
         if isinstance(self.__schema, SchemaABC):
@@ -174,7 +174,8 @@ class Relationship(BaseRelationship):
         # relationship. Unserialize it if we have a schema set; otherwise we
         # fall back below to old behaviour of only IDs.
         if 'attributes' in data and self.__schema:
-            return self.schema.load({'data': data}).data
+            result = self.schema.load({'data': data})
+            return result.data if _MARSHMALLOW_VERSION_INFO[0] < 3 else result
 
         return data.get('id')
 
@@ -186,7 +187,11 @@ class Relationship(BaseRelationship):
             required but unspecified.
         """
         if not isinstance(value, dict) or 'data' not in value:
-            raise ValidationError('Must include a `data` key')
+            # a relationships object does not need 'data' if 'links' is present
+            if 'links' in value:
+                return missing
+            else:
+                raise ValidationError('Must include a `data` key')
         return super(Relationship, self).deserialize(value['data'], attr, data)
 
     def _deserialize(self, value, attr, obj):
@@ -229,9 +234,13 @@ class Relationship(BaseRelationship):
 
     def _serialize_included(self, value):
         result = self.schema.dump(value)
-        if result.errors:
-            raise ValidationError(result.errors)
-        item = result.data['data']
+
+        if _MARSHMALLOW_VERSION_INFO[0] < 3:
+            data = result.data
+        else:
+            data = result
+
+        item = data['data']
         self.root.included_data[(item['type'], item['id'])] = item
         for key, value in iteritems(self.schema.included_data):
             self.root.included_data[key] = value
@@ -273,7 +282,10 @@ class Meta(Field):
 
     def __init__(self, **kwargs):
         super(Meta, self).__init__(**kwargs)
-        self.load_from = _META_LOAD_FROM
+        if _MARSHMALLOW_VERSION_INFO[0] < 3:
+            self.load_from = _META_LOAD_FROM
+        else:
+            self.data_key = _META_LOAD_FROM
 
     def _deserialize(self, value, attr, data):
         if isinstance(value, collections.Mapping):
