@@ -7,7 +7,7 @@ from marshmallow.utils import is_collection
 
 from .fields import BaseRelationship, Meta, _META_LOAD_FROM
 from .exceptions import IncorrectTypeError
-from .utils import resolve_params
+from .utils import resolve_params, _MARSHMALLOW_VERSION_INFO, get_dump_key
 
 TYPE = 'type'
 ID = 'id'
@@ -93,6 +93,7 @@ class Schema(ma.Schema):
             raise ValueError('Must specify `self_url` Meta option when '
                              '`self_url_kwargs` is specified')
         self.included_data = {}
+        self.meta_information = {}
 
     OPTIONS_CLASS = SchemaOpts
 
@@ -125,6 +126,7 @@ class Schema(ma.Schema):
         ret = self.format_items(data, many)
         ret = self.wrap_response(ret, many)
         ret = self.render_included_data(ret)
+        ret = self.render_meta_information(ret)
         return ret
 
     def render_included_data(self, data):
@@ -133,6 +135,12 @@ class Schema(ma.Schema):
         data['included'] = list(self.included_data.values())
         return data
 
+    def render_meta_information(self, data):
+        if not self.meta_information:
+            return data
+        data['meta'] = self.meta_information
+        return data
+
     def unwrap_item(self, item):
         if 'type' not in item:
             raise ma.ValidationError([
@@ -149,8 +157,8 @@ class Schema(ma.Schema):
         payload = self.dict_class()
         if 'id' in item:
             payload['id'] = item['id']
-        if 'meta' in item:
-            payload[_META_LOAD_FROM] = item['meta']
+        if self.meta_information:
+            payload[_META_LOAD_FROM] = self.meta_information
         for key, value in iteritems(item.get('attributes', {})):
             payload[key] = value
         for key, value in iteritems(item.get('relationships', {})):
@@ -195,11 +203,15 @@ class Schema(ma.Schema):
         return self.unwrap_item(data)
 
     def on_bind_field(self, field_name, field_obj):
-        """Schema hook override. When binding fields, set load_from to the
-        inflected form of field_name.
+        """Schema hook override. When binding fields, set ``data_key`` (on marshmallow 3) or
+        load_from (on marshmallow 2) to the inflected form of field_name.
         """
-        if not field_obj.load_from:
-            field_obj.load_from = self.inflect(field_name)
+        if _MARSHMALLOW_VERSION_INFO[0] < 3:
+            if not field_obj.load_from:
+                field_obj.load_from = self.inflect(field_name)
+        else:
+            if not field_obj.data_key:
+                field_obj.data_key = self.inflect(field_name)
         return None
 
     def _do_load(self, data, many=None, **kwargs):
@@ -214,9 +226,10 @@ class Schema(ma.Schema):
         # when processing relationships (``included`` is outside of the
         # ``data``).
         self.included_data = data.get('included', {})
+        self.meta_information = data.get('meta', {})
 
         try:
-            result, errors = super(Schema, self)._do_load(data, many, **kwargs)
+            result = super(Schema, self)._do_load(data, many, **kwargs)
         except ValidationError as err:  # strict mode
             error_messages = err.messages
             if '_schema' in error_messages:
@@ -225,11 +238,14 @@ class Schema(ma.Schema):
             err.messages = formatted_messages
             raise err
         else:
-            error_messages = errors
-            if '_schema' in error_messages:
-                error_messages = error_messages['_schema']
-            formatted_messages = self.format_errors(error_messages, many=many)
-        return result, formatted_messages
+            # On marshmallow 2, _do_load returns a tuple (load_data, errors)
+            if _MARSHMALLOW_VERSION_INFO[0] < 3:
+                data, error_messages = result
+                if '_schema' in error_messages:
+                    error_messages = error_messages['_schema']
+                formatted_messages = self.format_errors(error_messages, many=many)
+                return data, formatted_messages
+        return result
 
     def _extract_from_included(self, data):
         """Extract included data matching the items in ``data``.
@@ -318,7 +334,7 @@ class Schema(ma.Schema):
 
         # Get the schema attributes so we can confirm `dump-to` values exist
         attributes = {
-            (self.fields[field].dump_to or field): field
+            (get_dump_key(self.fields[field]) or field): field
             for field in self.fields
         }
 
@@ -327,9 +343,9 @@ class Schema(ma.Schema):
             if attribute == ID:
                 ret[ID] = value
             elif isinstance(self.fields[attribute], Meta):
-                if 'meta' not in ret:
-                    ret['meta'] = self.dict_class()
-                ret['meta'].update(value)
+                if not self.meta_information:
+                    self.meta_information = self.dict_class()
+                self.meta_information.update(value)
             elif isinstance(self.fields[attribute], BaseRelationship):
                 if value:
                     if 'relationships' not in ret:
